package net.wanji.opt.service.strategy.impl;

import cn.hutool.core.util.ObjectUtil;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import net.wanji.opt.dao.mapper.strategy.IdeaMapper;
import net.wanji.opt.dao.mapper.strategy.SceneStrategyIdeaMapper;
import net.wanji.opt.dto.IntegerIdsDTO;
import net.wanji.opt.dto.strategy.AddOrUpdateIdeaDTO;
import net.wanji.opt.dto.strategy.QueryIdeaDTO;
import net.wanji.opt.po.strategy.IdeaPO;
import net.wanji.opt.service.strategy.IdeaService;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

/**
 * @author Kent HAN
 * @date 2023/2/27 9:43
 */
@Service
public class IdeaServiceImpl implements IdeaService {
    private final IdeaMapper ideaMapper;
    private final SceneStrategyIdeaMapper sceneStrategyIdeaMapper;

    public IdeaServiceImpl(IdeaMapper ideaMapper, SceneStrategyIdeaMapper sceneStrategyIdeaMapper) {
        this.ideaMapper = ideaMapper;
        this.sceneStrategyIdeaMapper = sceneStrategyIdeaMapper;
    }

    @Override
    @Transactional
    public void addOrUpdateIdea(AddOrUpdateIdeaDTO addOrUpdateIdeaDTO) {
        Integer id = addOrUpdateIdeaDTO.getId();
        if (ObjectUtil.isEmpty(id) || id == 0) {
            // 不传ID为新增
            IdeaPO ideaPO = new IdeaPO();
            fillData(addOrUpdateIdeaDTO, ideaPO);
            ideaMapper.insertOne(ideaPO);
        } else {
            // 传ID为修改
            IdeaPO ideaPO = new IdeaPO();
            ideaPO.setId(id);
            fillData(addOrUpdateIdeaDTO, ideaPO);
            ideaMapper.updateOne(ideaPO);
        }
    }

    private void fillData(AddOrUpdateIdeaDTO addOrUpdateIdeaDTO, IdeaPO ideaPO) {
        ideaPO.setIdeaCode(addOrUpdateIdeaDTO.getIdeaCode());
        ideaPO.setIdeaName(addOrUpdateIdeaDTO.getIdeaName());
        ideaPO.setIdeaTarget(addOrUpdateIdeaDTO.getIdeaTarget());
        ideaPO.setIdeaDetail(addOrUpdateIdeaDTO.getIdeaDetail());
    }

    @Override
    public void deleteIdea(IntegerIdsDTO integerIdsDTO) {
        List<Integer> ids = integerIdsDTO.getIds();
        ideaMapper.deleteByIds(ids);
        // 删除关系表数据
        sceneStrategyIdeaMapper.deleteByIdeaIds(ids);
    }

    @Override
    public PageInfo<IdeaPO> queryIdea(QueryIdeaDTO queryIdeaDTO) {
        Integer pageNum = queryIdeaDTO.getPageNum();
        Integer pageSize = queryIdeaDTO.getPageSize();
        PageHelper.startPage(pageNum, pageSize);
        List<IdeaPO> ideaPOList = ideaMapper.selectByIdeaNameAndIdeaTarget(queryIdeaDTO);
        PageInfo<IdeaPO> ideaPOPageInfo = new PageInfo<>(ideaPOList);
        return ideaPOPageInfo;
    }
}
